@@ -20,21 +20,32 @@
 
 #include "config.h"
 
-#include <cmath>
-#include <cstdlib>
-#include <array>
-#include <complex>
 #include <algorithm>
+#include <array>
+#include <cmath>
+#include <complex>
+#include <cstdlib>
+#include <iterator>
 
-#include "al/auxeffectslot.h"
-#include "alcmain.h"
-#include "alcontext.h"
-#include "alu.h"
-
+#include "alc/effects/base.h"
 #include "alcomplex.h"
+#include "almalloc.h"
+#include "alnumbers.h"
+#include "alnumeric.h"
+#include "alspan.h"
+#include "core/bufferline.h"
+#include "core/context.h"
+#include "core/devformat.h"
+#include "core/device.h"
+#include "core/effectslot.h"
+#include "core/mixer.h"
+#include "core/mixer/defs.h"
+#include "intrusive_ptr.h"
+
 
 namespace {
 
+using uint = unsigned int;
 using complex_d = std::complex<double>;
 
 #define HIL_SIZE 1024
@@ -50,7 +61,7 @@ std::array<double,HIL_SIZE> InitHannWindow()
     /* Create lookup table of the Hann window for the desired size, i.e. HIL_SIZE */
     for(size_t i{0};i < HIL_SIZE>>1;i++)
     {
-        constexpr double scale{al::MathDefs<double>::Pi() / double{HIL_SIZE}};
+        constexpr double scale{al::numbers::pi / double{HIL_SIZE}};
         const double val{std::sin(static_cast<double>(i+1) * scale)};
         ret[i] = ret[HIL_SIZE-1-i] = val * val;
     }
@@ -62,8 +73,9 @@ alignas(16) const std::array<double,HIL_SIZE> HannWindow = InitHannWindow();
 struct FshifterState final : public EffectState {
     /* Effect parameters */
     size_t mCount{};
-    ALuint mPhaseStep[2]{};
-    ALuint mPhase[2]{};
+    size_t mPos{};
+    uint mPhaseStep[2]{};
+    uint mPhase[2]{};
     double mSign[2]{};
 
     /* Effects buffers */
@@ -71,9 +83,9 @@ struct FshifterState final : public EffectState {
     complex_d mOutFIFO[HIL_STEP]{};
     complex_d mOutputAccum[HIL_SIZE]{};
     complex_d mAnalytic[HIL_SIZE]{};
-    complex_d mOutdata[BUFFERSIZE]{};
+    complex_d mOutdata[BufferLineSize]{};
 
-    alignas(16) float mBufferOut[BUFFERSIZE]{};
+    alignas(16) float mBufferOut[BufferLineSize]{};
 
     /* Effect gains for each output channel */
     struct {
@@ -82,17 +94,20 @@ struct FshifterState final : public EffectState {
     } mGains[2];
 
 
-    void deviceUpdate(const ALCdevice *device) override;
-    void update(const ALCcontext *context, const ALeffectslot *slot, const EffectProps *props, const EffectTarget target) override;
-    void process(const size_t samplesToDo, const al::span<const FloatBufferLine> samplesIn, const al::span<FloatBufferLine> samplesOut) override;
+    void deviceUpdate(const DeviceBase *device, const Buffer &buffer) override;
+    void update(const ContextBase *context, const EffectSlot *slot, const EffectProps *props,
+        const EffectTarget target) override;
+    void process(const size_t samplesToDo, const al::span<const FloatBufferLine> samplesIn,
+        const al::span<FloatBufferLine> samplesOut) override;
 
     DEF_NEWDEL(FshifterState)
 };
 
-void FshifterState::deviceUpdate(const ALCdevice*)
+void FshifterState::deviceUpdate(const DeviceBase*, const Buffer&)
 {
     /* (Re-)initializing parameters and clear the buffers. */
-    mCount = FIFO_LATENCY;
+    mCount = 0;
+    mPos = FIFO_LATENCY;
 
     std::fill(std::begin(mPhaseStep),   std::end(mPhaseStep),   0u);
     std::fill(std::begin(mPhase),       std::end(mPhase),       0u);
@@ -109,24 +124,23 @@ void FshifterState::deviceUpdate(const ALCdevice*)
     }
 }
 
-void FshifterState::update(const ALCcontext *context, const ALeffectslot *slot, const EffectProps *props, const EffectTarget target)
+void FshifterState::update(const ContextBase *context, const EffectSlot *slot,
+    const EffectProps *props, const EffectTarget target)
 {
-    const ALCdevice *device{context->mDevice.get()};
+    const DeviceBase *device{context->mDevice};
 
     const float step{props->Fshifter.Frequency / static_cast<float>(device->Frequency)};
     mPhaseStep[0] = mPhaseStep[1] = fastf2u(minf(step, 1.0f) * MixerFracOne);
 
     switch(props->Fshifter.LeftDirection)
     {
-    case AL_FREQUENCY_SHIFTER_DIRECTION_DOWN:
+    case FShifterDirection::Down:
         mSign[0] = -1.0;
         break;
-
-    case AL_FREQUENCY_SHIFTER_DIRECTION_UP:
+    case FShifterDirection::Up:
         mSign[0] = 1.0;
         break;
-
-    case AL_FREQUENCY_SHIFTER_DIRECTION_OFF:
+    case FShifterDirection::Off:
         mPhase[0]     = 0;
         mPhaseStep[0] = 0;
         break;
@@ -134,15 +148,13 @@ void FshifterState::update(const ALCcontext *context, const ALeffectslot *slot,
 
     switch(props->Fshifter.RightDirection)
     {
-    case AL_FREQUENCY_SHIFTER_DIRECTION_DOWN:
+    case FShifterDirection::Down:
         mSign[1] = -1.0;
         break;
-
-    case AL_FREQUENCY_SHIFTER_DIRECTION_UP:
+    case FShifterDirection::Up:
         mSign[1] = 1.0;
         break;
-
-    case AL_FREQUENCY_SHIFTER_DIRECTION_OFF:
+    case FShifterDirection::Off:
         mPhase[1]     = 0;
         mPhaseStep[1] = 0;
         break;
@@ -152,57 +164,60 @@ void FshifterState::update(const ALCcontext *context, const ALeffectslot *slot,
     const auto rcoeffs = CalcDirectionCoeffs({ 1.0f, 0.0f, 0.0f}, 0.0f);
 
     mOutTarget = target.Main->Buffer;
-    ComputePanGains(target.Main, lcoeffs.data(), slot->Params.Gain, mGains[0].Target);
-    ComputePanGains(target.Main, rcoeffs.data(), slot->Params.Gain, mGains[1].Target);
+    ComputePanGains(target.Main, lcoeffs.data(), slot->Gain, mGains[0].Target);
+    ComputePanGains(target.Main, rcoeffs.data(), slot->Gain, mGains[1].Target);
 }
 
 void FshifterState::process(const size_t samplesToDo, const al::span<const FloatBufferLine> samplesIn, const al::span<FloatBufferLine> samplesOut)
 {
     for(size_t base{0u};base < samplesToDo;)
     {
-        size_t todo{minz(HIL_SIZE-mCount, samplesToDo-base)};
+        size_t todo{minz(HIL_STEP-mCount, samplesToDo-base)};
 
         /* Fill FIFO buffer with samples data */
+        const size_t pos{mPos};
         size_t count{mCount};
         do {
-            mInFIFO[count] = samplesIn[0][base];
-            mOutdata[base] = mOutFIFO[count-FIFO_LATENCY];
+            mInFIFO[pos+count] = samplesIn[0][base];
+            mOutdata[base] = mOutFIFO[count];
             ++base; ++count;
         } while(--todo);
         mCount = count;
 
         /* Check whether FIFO buffer is filled */
-        if(mCount < HIL_SIZE) break;
-        mCount = FIFO_LATENCY;
+        if(mCount < HIL_STEP) break;
+        mCount = 0;
+        mPos = (mPos+HIL_STEP) & (HIL_SIZE-1);
 
         /* Real signal windowing and store in Analytic buffer */
-        for(size_t k{0};k < HIL_SIZE;k++)
-            mAnalytic[k] = mInFIFO[k]*HannWindow[k];
+        for(size_t src{mPos}, k{0u};src < HIL_SIZE;++src,++k)
+            mAnalytic[k] = mInFIFO[src]*HannWindow[k];
+        for(size_t src{0u}, k{HIL_SIZE-mPos};src < mPos;++src,++k)
+            mAnalytic[k] = mInFIFO[src]*HannWindow[k];
 
         /* Processing signal by Discrete Hilbert Transform (analytical signal). */
         complex_hilbert(mAnalytic);
 
         /* Windowing and add to output accumulator */
-        for(size_t k{0};k < HIL_SIZE;k++)
-            mOutputAccum[k] += 2.0/OVERSAMP*HannWindow[k]*mAnalytic[k];
+        for(size_t dst{mPos}, k{0u};dst < HIL_SIZE;++dst,++k)
+            mOutputAccum[dst] += 2.0/OVERSAMP*HannWindow[k]*mAnalytic[k];
+        for(size_t dst{0u}, k{HIL_SIZE-mPos};dst < mPos;++dst,++k)
+            mOutputAccum[dst] += 2.0/OVERSAMP*HannWindow[k]*mAnalytic[k];
 
-        /* Shift accumulator, input & output FIFO */
-        std::copy_n(mOutputAccum, HIL_STEP, mOutFIFO);
-        auto accum_iter = std::copy(std::begin(mOutputAccum)+HIL_STEP, std::end(mOutputAccum),
-            std::begin(mOutputAccum));
-        std::fill(accum_iter, std::end(mOutputAccum), complex_d{});
-        std::copy(std::begin(mInFIFO)+HIL_STEP, std::end(mInFIFO), std::begin(mInFIFO));
+        /* Copy out the accumulated result, then clear for the next iteration. */
+        std::copy_n(mOutputAccum + mPos, HIL_STEP, mOutFIFO);
+        std::fill_n(mOutputAccum + mPos, HIL_STEP, complex_d{});
     }
 
     /* Process frequency shifter using the analytic signal obtained. */
     float *RESTRICT BufferOut{mBufferOut};
-    for(ALsizei c{0};c < 2;++c)
+    for(int c{0};c < 2;++c)
     {
-        const ALuint phase_step{mPhaseStep[c]};
-        ALuint phase_idx{mPhase[c]};
+        const uint phase_step{mPhaseStep[c]};
+        uint phase_idx{mPhase[c]};
         for(size_t k{0};k < samplesToDo;++k)
         {
-            const double phase{phase_idx * ((1.0/MixerFracOne) * al::MathDefs<double>::Tau())};
+            const double phase{phase_idx * (al::numbers::pi*2.0 / MixerFracOne)};
             BufferOut[k] = static_cast<float>(mOutdata[k].real()*std::cos(phase) +
                 mOutdata[k].imag()*std::sin(phase)*mSign[c]);
 
@@ -218,102 +233,11 @@ void FshifterState::process(const size_t samplesToDo, const al::span<const Float
 }
 
 
-void Fshifter_setParamf(EffectProps *props, ALenum param, float val)
-{
-    switch(param)
-    {
-    case AL_FREQUENCY_SHIFTER_FREQUENCY:
-        if(!(val >= AL_FREQUENCY_SHIFTER_MIN_FREQUENCY && val <= AL_FREQUENCY_SHIFTER_MAX_FREQUENCY))
-            throw effect_exception{AL_INVALID_VALUE, "Frequency shifter frequency out of range"};
-        props->Fshifter.Frequency = val;
-        break;
-
-    default:
-        throw effect_exception{AL_INVALID_ENUM, "Invalid frequency shifter float property 0x%04x",
-            param};
-    }
-}
-void Fshifter_setParamfv(EffectProps *props, ALenum param, const float *vals)
-{ Fshifter_setParamf(props, param, vals[0]); }
-
-void Fshifter_setParami(EffectProps *props, ALenum param, int val)
-{
-    switch(param)
-    {
-    case AL_FREQUENCY_SHIFTER_LEFT_DIRECTION:
-        if(!(val >= AL_FREQUENCY_SHIFTER_MIN_LEFT_DIRECTION && val <= AL_FREQUENCY_SHIFTER_MAX_LEFT_DIRECTION))
-            throw effect_exception{AL_INVALID_VALUE,
-                "Frequency shifter left direction out of range"};
-        props->Fshifter.LeftDirection = val;
-        break;
-
-    case AL_FREQUENCY_SHIFTER_RIGHT_DIRECTION:
-        if(!(val >= AL_FREQUENCY_SHIFTER_MIN_RIGHT_DIRECTION && val <= AL_FREQUENCY_SHIFTER_MAX_RIGHT_DIRECTION))
-            throw effect_exception{AL_INVALID_VALUE,
-                "Frequency shifter right direction out of range"};
-        props->Fshifter.RightDirection = val;
-        break;
-
-    default:
-        throw effect_exception{AL_INVALID_ENUM,
-            "Invalid frequency shifter integer property 0x%04x", param};
-    }
-}
-void Fshifter_setParamiv(EffectProps *props, ALenum param, const int *vals)
-{ Fshifter_setParami(props, param, vals[0]); }
-
-void Fshifter_getParami(const EffectProps *props, ALenum param, int *val)
-{
-    switch(param)
-    {
-    case AL_FREQUENCY_SHIFTER_LEFT_DIRECTION:
-        *val = props->Fshifter.LeftDirection;
-        break;
-    case AL_FREQUENCY_SHIFTER_RIGHT_DIRECTION:
-        *val = props->Fshifter.RightDirection;
-        break;
-    default:
-        throw effect_exception{AL_INVALID_ENUM,
-            "Invalid frequency shifter integer property 0x%04x", param};
-    }
-}
-void Fshifter_getParamiv(const EffectProps *props, ALenum param, int *vals)
-{ Fshifter_getParami(props, param, vals); }
-
-void Fshifter_getParamf(const EffectProps *props, ALenum param, float *val)
-{
-    switch(param)
-    {
-    case AL_FREQUENCY_SHIFTER_FREQUENCY:
-        *val = props->Fshifter.Frequency;
-        break;
-
-    default:
-        throw effect_exception{AL_INVALID_ENUM, "Invalid frequency shifter float property 0x%04x",
-            param};
-    }
-}
-void Fshifter_getParamfv(const EffectProps *props, ALenum param, float *vals)
-{ Fshifter_getParamf(props, param, vals); }
-
-DEFINE_ALEFFECT_VTABLE(Fshifter);
-
-
 struct FshifterStateFactory final : public EffectStateFactory {
-    EffectState *create() override { return new FshifterState{}; }
-    EffectProps getDefaultProps() const noexcept override;
-    const EffectVtable *getEffectVtable() const noexcept override { return &Fshifter_vtable; }
+    al::intrusive_ptr<EffectState> create() override
+    { return al::intrusive_ptr<EffectState>{new FshifterState{}}; }
 };
 
-EffectProps FshifterStateFactory::getDefaultProps() const noexcept
-{
-    EffectProps props{};
-    props.Fshifter.Frequency      = AL_FREQUENCY_SHIFTER_DEFAULT_FREQUENCY;
-    props.Fshifter.LeftDirection  = AL_FREQUENCY_SHIFTER_DEFAULT_LEFT_DIRECTION;
-    props.Fshifter.RightDirection = AL_FREQUENCY_SHIFTER_DEFAULT_RIGHT_DIRECTION;
-    return props;
-}
-
 } // namespace
 
 EffectStateFactory *FshifterStateFactory_getFactory()
